/// <reference types="react" />
import { Action } from 'popmotion';
import { ComponentType } from 'react';
import { CSSProperties } from 'react';
import { DetailedHTMLFactory } from 'react';
import { FrameData } from 'framesync';
import * as React from 'react';
import { ReactHTML } from 'react';
import { ReactNode } from 'react';
import { ReactSVG } from 'react';
import { Ref } from 'react';
import { RefObject } from 'react';
import { SVGAttributes } from 'react';

declare type ActionFactory = (actionConfig: PopmotionTransitionProps) => Action;

declare class AnimationControls {
    private hasMounted;
    private defaultTransition;
    private pendingAnimations;
    private componentControls;
    private variants;
    setVariants(variants: Variants): void;
    setDefaultTransition(transition: Transition): void;
    subscribe(controls: ComponentAnimationControls): () => boolean;
    start(definition: Variant | string): Promise<any>;
    stop(): void;
    mount(): void;
    unmount(): void;
}

declare type AnimationDefinition = VariantLabels | TargetAndTransition | TargetResolver;

declare type AnimationOptions = {
    delay?: number;
    priority?: number;
};

/**
 * @public
 */
export declare interface AnimationProps {
    /**
     * Values to animate to, variant label(s), or `AnimationControls`
     *
     * ```jsx
     * // As values
     * <motion.div animate={{ opacity: 1 }} />
     *
     * // As variant
     * <motion.div animate="visible" variants={variants} />
     *
     * // Multiple variants
     * <motion.div animate={["visible", "active"]} variants={variants} />
     *
     * // `AnimationControls`
     * <motion.div animate={animation} />
     * ```
     */
    animate?: AnimationControls | TargetAndTransition | VariantLabels;
    /**
     * Object of labelled variants.
     *
     * ```jsx
     * function App() {
     *   const variants = {
     *     active: {
     *       backgroundColor: '#f00'
     *    },
     *     inactive: {
     *       backgroundColor: '#fff',
     *       transition: { duration: 2 }
     *     }
     *   }
     *
     *   return <motion.div variants={variants} animate="active" />
     * }
     * ```
     */
    variants?: Variants;
    /**
     * Default transition, to fall back on if no `transition` is defined in `animate`.
     *
     * ```jsx
     * function App() {
     *   const transition = {
     *     type: 'spring',
     *     damping: 10,
     *     stiffness: 100
     *   }
     *
     *   return <motion.div transition={transition} animate={{ scale: 1.2 }} />
     * }
     * ```
     */
    transition?: Transition;
}

declare class ComponentAnimationControls<P = {}> {
    private props;
    private values;
    private ref;
    private variants;
    private baseTarget;
    private overrides;
    private activeOverrides;
    private resolvedOverrides;
    private defaultTransition?;
    private children?;
    private isAnimating;
    private customStyles?;
    constructor(values: MotionValuesMap, ref: RefObject<Element>, customStyles?: CustomStyleMap);
    setProps(props: P): void;
    setVariants(variants?: Variants): void;
    setDefaultTransition(transition?: Transition): void;
    setValues(target: Target, isActive?: Set<string>): void;
    checkForNewValues(target: Target): void;
    resolveVariant(variant?: Variant): {
        target?: Target;
        transition?: Transition;
        transitionEnd?: Target;
    };
    getHighestPriority(): number;
    setOverride(definition: AnimationDefinition, overrideIndex: number): void;
    startOverride(overrideIndex: number): Promise<any> | undefined;
    clearOverride(overrideIndex: number): void;
    apply(definition: VariantLabels | TargetAndTransition): void;
    private applyVariantLabels;
    start(definition: AnimationDefinition, opts?: AnimationOptions): Promise<any>;
    isHighestPriority(priority: number): boolean;
    private animate;
    private animateVariantLabels;
    private animateVariant;
    private animateChildren;
    private checkOverrideIsAnimating;
    private resetIsAnimating;
    stop(): void;
    addChild(controls: ComponentAnimationControls): void;
    removeChild(controls: ComponentAnimationControls): void;
    resetChildren(): void;
}

declare type Config<T> = {
    transformer?: Transformer<T>;
    parent?: MotionValue<T>;
};

declare interface ControlsProp {
    controls?: ComponentAnimationControls;
}

/**
 * @internal
 */
export declare const createMotionComponent: <P extends {}>(Component: string | React.ComponentClass<P, any> | React.FunctionComponent<P>) => React.ForwardRefExoticComponent<Pick<P & MotionProps, string | number | Exclude<keyof P, "ref">> & React.RefAttributes<Element>>;

declare type CSSPropertiesWithoutTransition = Omit_2<CSSProperties, "transition">;

/**
 * A factory to create motion-optimised versions of existing React DOM components.
 *
 * @public
 */
export declare type CustomMotionComponent = {
    custom: typeof createMotionComponent;
};

declare interface CustomProperties {
    size?: string | number;
    image?: string;
}

declare type CustomStyle = {
    transformToStyles: (v: number | string) => {
        [key: string]: number | string;
    };
    motionEnabled?: boolean;
};

declare type CustomStyleMap = {
    [key: string]: CustomStyle;
};

declare type Cycle = (i?: number) => void;

declare type CycleState<T> = [T, Cycle];

/**
 * @public
 */
export declare interface DraggableProps {
    /**
     * Enable dragging for this element. Set to `false` by default.
     *
     * @remarks
     * Set "x" or "y" to only drag in a specific direction
     * Set "lockDirection" to lock dragging into the initial direction
     *
     * ```jsx
     * function App() {
     *   return <motion.div dragEnabled="x" />
     * }
     * ```
     */
    dragEnabled?: boolean | "x" | "y" | "lockDirection";
    /**
     * Allows drag gesture propagation to child components. Set to `false` by
     * default.
     *
     * ```jsx
     * function App() {
     *   return <motion.div dragEnabled="x" dragPropagation />
     * }
     * ```
     */
    dragPropagation?: boolean;
    /**
     * An object of optional `top`, `left`, `right`, `bottom` pixel values,
     * beyond which dragging is constrained
     *
     * ```jsx
     * function App() {
     *   return (
     *     <motion.div
     *       dragEnabled="x"
     *       dragConstraints={{ left: 0, right: 300 }}
     *     />
     *   )
     * }
     * ```
     */
    dragConstraints?: false | {
        top?: number;
        right?: number;
        bottom?: number;
        left?: number;
    };
    /**
     * The degree of movement allowed outside constraints. 0 = no movement, 1 =
     * full movement. Set to `0.5` by default.
     *
     * ```jsx
     * function App() {
     *   return (
     *     <motion.div
     *       dragEnabled
     *       dragConstraints={{ left: 0, right: 300 }}
     *       dragElastic={0.2}
     *     />
     *   )
     * }
     * ```
     */
    dragElastic?: boolean | number;
    /**
     * Apply momentum from the pan gesture to the component when dragging
     * finishes. Set to `true` by default.
     *
     * ```jsx
     * function App() {
     *   return (
     *     <motion.div
     *       dragEnabled
     *       dragConstraints={{ left: 0, right: 300 }}
     *       dragMomentum={false}
     *     />
     *   )
     * }
     * ```
     */
    dragMomentum?: boolean;
    /**
     * Callback that fires when dragging starts.
     *
     * ```jsx
     * function App() {
     *   const onDragStart = (event, { point, delta, offset, velocity }) => {
     *     console.log(`Dragging started at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div dragEnabled onDragStart={onDragStart} />
     * }
     * ```
     */
    onDragStart?(e: MouseEvent | TouchEvent): void;
    /**
     * Callback that fires when dragging ends.
     *
     * ```jsx
     * function App() {
     *   const onDragEnd = (event, { point, delta, offset, velocity }) => {
     *     console.log(`Dragging ended at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div dragEnabled onDragEnd={onDragEnd} />
     * }
     * ```
     */
    onDragEnd?(e: MouseEvent | TouchEvent): void;
    /**
     * Callback that fires when the component is dragged.
     *
     * ```jsx
     * function App() {
     *   const onDrag = (event, { point, delta, offset, velocity }) => {
     *     console.log(`Drag velocity is ${velocity.x} ${velocity.y}`)
     *   }
     *
     *   return <motion.div dragEnabled onDrag={onDrag} />
     * }
     * ```
     */
    onDrag?(e: MouseEvent | TouchEvent, info: PanInfo): void;
    /**
     * Callback that fires a drag direction is determined.
     *
     * ```jsx
     * function App() {
     *   const onDirectionLock = (axis) => {
     *     console.log(`Dragging locked to the ${axis} axis`)
     *   }
     *
     *   return <motion.div dragEnabled onDirectionLock={onDirectionLock} />
     * }
     * ```
     */
    onDirectionLock?(axis: "x" | "y"): void;
}

declare type Easing = [number, number, number, number] | "linear" | "easeIn" | "easeOut" | "easeInOut" | "circIn" | "circOut" | "circInOut" | "backIn" | "backOut" | "backInOut" | "anticipate" | EasingFunction;

declare type EasingFunction = (v: number) => number;

declare type EventHandler = (event: Event, info: EventInfo) => void;

declare interface EventInfo {
    point: Point;
}

/**
 * @public
 */
export declare type GestureHandlers = PanHandlers & TapHandlers & HoverHandlers;

/**
 * @public
 */
export declare interface HoverHandlers {
    /**
     * Properties or variant label to animate to while the hover gesture is recognised.
     *
     * ```jsx
     * function App() {
     *   return <motion.div hover={{ scale: 1.2 }} />
     * }
     * ```
     */
    hover?: string | TargetAndTransition;
    /**
     * Callback that fires when pointer starts hovering over the component.
     *
     * ```jsx
     * function App() {
     *   const onHoverStart = (event) => {
     *      console.log('Pointer is hovering over this Frame')
     *   }
     *
     *   return <motion.div onHoverStart={onHoverStart} />
     * }
     * ```
     */
    onHoverStart?(event: MouseEvent): void;
    /**
     * Callback that fires when pointer stops hovering over the component.
     *
     * ```jsx
     * function App() {
     *   const onHoverEnd = (event) => {
     *      console.log('Pointer stopped hovering over this Frame')
     *   }
     *
     *   return <motion.div onHoverEnd={onHoverEnd} />
     * }
     * ```
     */
    onHoverEnd?(event: MouseEvent): void;
}

declare type HTMLElements = UnionStringArray<typeof htmlElements>;

/**
 * @internal
 */
export declare const htmlElements: ("object" | "track" | "progress" | "a" | "abbr" | "address" | "area" | "article" | "aside" | "audio" | "b" | "base" | "bdo" | "blockquote" | "body" | "br" | "button" | "canvas" | "caption" | "cite" | "code" | "col" | "colgroup" | "data" | "datalist" | "dd" | "del" | "details" | "dfn" | "dialog" | "div" | "dl" | "dt" | "em" | "embed" | "fieldset" | "figcaption" | "figure" | "footer" | "form" | "h1" | "h2" | "h3" | "h4" | "h5" | "h6" | "head" | "header" | "hgroup" | "hr" | "html" | "i" | "iframe" | "img" | "input" | "ins" | "kbd" | "label" | "legend" | "li" | "link" | "map" | "mark" | "menu" | "meta" | "meter" | "nav" | "noscript" | "ol" | "optgroup" | "option" | "output" | "p" | "param" | "picture" | "pre" | "q" | "rt" | "ruby" | "s" | "samp" | "script" | "section" | "select" | "small" | "source" | "span" | "strong" | "style" | "sub" | "sup" | "table" | "tbody" | "td" | "textarea" | "tfoot" | "th" | "thead" | "time" | "title" | "tr" | "u" | "ul" | "var" | "video" | "wbr" | "menuitem" | "bdi" | "big" | "keygen" | "main" | "rp" | "summary" | "webview")[];

/**
 * Motion-optimised versions of React's HTML components.
 *
 * @public
 */
export declare type HTMLMotionComponents = {
    [K in HTMLElements]: ComponentType<Omit<UnwrapFactory<ReactHTML[K]>, "style"> & MotionProps>;
};

/**
 * An animation that decelerates a value based on its initial velocity,
 * usually used to implement inertial scrolling.
 *
 * Optionally, `min` and `max` boundaries can be defined, and inertia
 * will snap to these with a spring animation.
 *
 * This animation will automatically precalculate a target value,
 * which can be modified with the `modifyTarget` property.
 * This allows you to add snap-to-grid or similar functionality.
 *
 * @public
 */
export declare interface Inertia {
    /**
     * Set `type` to animate using the inertia animation. Set to `'tween'` by
     * default.
     *
     * @remarks
     *
     * This can be used for natural deceleration, for instance with momentum scrolling.
     *
     * @public
     */
    type: "inertia";
    /**
     * A function that receives the automatically-calculated target and returns a new one. Useful for snapping the target to a grid.
     *
     * @public
     */
    modifyTarget?(v: number): number;
    /**
     * If `min` or `max` is set, this affects the stiffness of the bounce
     * spring. Higher values will create more sudden movement. Set to `500` by
     * default.
     *
     * @public
     */
    bounceStiffness?: number;
    /**
     * If `min` or `max` is set, this affects the damping of the bounce spring.
     * If set to `0`, spring will oscillate indefinitely. Set to `10` by
     * default.
     * @public
     */
    bounceDamping?: number;
    /**
     * Higher power = further target. Set to `0.8` by default.
     * @public
     */
    power?: number;
    /**
     * Adjusting the time constant will change the duration of the
     * deceleration, thereby affecting its feel. Set to `700` by default.
     *
     * @public
     */
    timeConstant?: number;
    /**
     * End animation if distance to to is below this value and speed is below
     * `restSpeed`. When animation ends, spring gets “snapped” to to. Set to
     * `0.01` by default.
     * @public
     */
    restDelta?: number;
    /**
     * Minimum constraint. If set, the value will "bump" against this value (or immediately spring to it if the animation starts as less than this value).
     * @public
     */
    min?: number;
    /**
     * Maximum constraint. If set, the value will "bump" against this value (or immediately spring to it if the animation starts as more than this value).
     * @public
     */
    max?: number;
    /**
     * The value to animate from. By default, this is the current state of the `MotionValue`.
     * @public
     */
    from?: number | string;
    /**
     * The initial velocity of the animation. By default this is the current velocity of the `MotionValue`.
     * @public
     */
    velocity?: number;
    /**
     * @internal
     */
    delay?: number;
}

/**
 * @internal
 */
declare interface Just {
    type: "just";
    from?: number | string;
    delay?: number;
    velocity?: number;
}

/**
 * Keyframes tweens between multiple `values`.
 *
 * These tweens can be arranged using the `duration`, `easings`, and `times` properties.
 *
 * @internalremarks
 * We could possibly make the `type` property redundant, if not for all animations
 * then for this one quite easily.
 *
 * @public
 */
export declare interface Keyframes {
    /**
     * Set `type` to `'keyframes'` to animate using the keyframes animation.
     * Set to `'tween'` by default.
     *
     * @remarks
     *
     * This can be used to animate between a series of values.
     *
     * @public
     */
    type: "keyframes";
    /**
     * An array of values to animate between.
     *
     * ```jsx
     * const transition = {
     *   type: 'keyframes',
     *   backgroundColor: {
     *     values: ['#0f0', '#f00', '#00f']
     *   }
     * }
     * ```
     *
     * @public
     */
    values: number[] | string[];
    /**
     * An array of numbers between 0 and 1, where `1` represents the `total` duration.
     *
     * Each value represents at which point during the animation each item in `values` should be hit, so the array should be the same length as `values`.
     *
     * Defaults to an array of evenly-spread durations.
     *
     * @public
     */
    times: number[];
    /**
     * An array of easing functions for each generated tween, or a single easing function applied to all tweens.
     *
     * This array should be one item less than `values`, as these easings apply to the transitions *between* the `values`.
     *
     * ```jsx
     * const transition = {
     *   type: 'keyframes',
     *   backgroundColor: {
     *     values: ['#0f0', '#f00', '#00f'],
     *     easings: ['circIn', 'circOut']
     *   }
     * }
     * ```
     *
     * @public
     */
    easings?: Easing[];
    /**
     * @internal
     */
    elapsed?: number;
    /**
     * The total duration of the animation. Set to `0.3` by default.
     *
     * @public
     */
    duration?: number;
    /**
     * The number of times to loop the animation.
     *
     * Set to `Infinity` for perpetual looping.
     *
     * @public
     */
    loop?: number;
    /**
     * The number of times to flip the animation by swapping the `to` and `from` values.
     *
     * Set to `Infinity` for perpetual flipping.
     *
     * @public
     */
    flip?: number;
    /**
     * The number of times to reverse the animation.
     *
     * Set to `Infinity` for perpetual reversing.
     *
     * @public
     */
    yoyo?: number;
    /**
     * @internal
     */
    from?: number | string;
    /**
     * @internal
     */
    to?: number | string;
    /**
     * @internal
     */
    velocity?: number;
    /**
     * @internal
     */
    delay?: number;
}

declare type MakeMotion<T> = {
    [K in keyof T]: T[K] | MotionValue<T[K]>;
};

declare type MapOptions = {
    clamp: true;
};

/**
 * HTML & SVG components, optimised for use with gestures and animation. These can be used as
 * drop-in replacements for any HTML & SVG component, all CSS & SVG properties are supported.
 *
 * ```jsx
 * <motion.div animate={{ x: 100 }} />
 *
 * <motion.p animate={{ height: 200 }} />
 *
 * <svg><motion.circle r={10} animate={{ r: 20 }} /></svg>
 * ```
 *
 * @public
 */
export declare const motion: MotionComponents;

/**
 * @public
 */
export declare interface MotionAdvancedProps {
    /**
     * Set to `false` to prevent inheriting variant changes from a parent
     * `motion` component.
     */
    inherit?: boolean;
    /**
     * Set to `true` to block rendering motion values (`animate`, gestures etc)
     * on the component.
     *
     * This can be used to temporarily disable animations for performance reasons.
     */
    static?: boolean;
}

/**
 * @public
 */
export declare interface MotionCallbacks {
    /**
     * Callback with latest motion values, fired max once per frame.
     *
     * ```jsx
     * function App() {
     *   const onUpdate = ({ x, opacity }) => {
     *     console.log(`Latest values: ${x} ${opacity}`)
     *   }
     *
     *   return <motion.div animate={{ x: 100, opacity: 0 }} onUpdate={onUpdate} />
     * }
     * ```
     */
    onUpdate?(latest: {
        [key: string]: string | number;
    }): void;
    /**
     * Callback when animation defined in `animate` is complete.
     *
     * ```jsx
     * function App() {
     *   const onComplete = () => {
     *     console.log(`Animation has completed`)
     *   }
     *
     *   return <motion.div animate={{ x: 100 }} onAnimationComplete={onComplete} />
     * }
     * ```
     */
    onAnimationComplete?(): void;
}

/**
 * Motion-optimised versions of React DOM components.
 *
 * ```jsx
 * <motion.div />
 * <motion.circle />
 * const Component = motion.custom(ExistingComponent)
 * ```
 *
 * @public
 */
export declare type MotionComponents = CustomMotionComponent & HTMLMotionComponents & SVGMotionComponents;

/**
 * @internal
 */
export declare const MotionContext: import("react").Context<MotionContextProps>;

declare type MotionContextProps = {
    controls?: ComponentAnimationControls;
    initial?: VariantLabels;
    dragging?: boolean;
    static?: boolean;
};

declare type MotionCSS = MakeMotion<Omit_2<CSSProperties, "rotate" | "scale" | "perspective">>;

declare type MotionCustom = MakeMotion<CustomProperties>;

/**
 * @internal
 */
export declare const MotionPluginContext: React.Context<Partial<MotionPlugins>>;

declare interface MotionPluginProps extends MotionPlugins {
    children?: ReactNode;
}

/**
 * @internal
 * @internalremarks For now I think this should remain a private API for our own use
 * until we can figure out a nicer way of allowing people to add these
 */
export declare function MotionPlugins({ children, ...props }: MotionPluginProps): JSX.Element;

export declare interface MotionPlugins {
    transformPagePoint?: (point: Point) => Point;
    customStyles?: CustomStyleMap;
}

/**
 * Props for `motion` components.
 *
 * @public
 */
export declare interface MotionProps extends AnimationProps, MotionCallbacks, GestureHandlers, DraggableProps, MotionAdvancedProps {
    /**
     * Properties, variant label or array of variant labels to start in
     *
     * ```jsx
     * // As values
     * <motion.div initial={{ opacity: 1 }} />
     *
     * // As variant
     * <motion.div initial="visible" variants={variants} />
     *
     * // Multiple variants
     * <motion.div initial={["visible", "active"]} variants={variants} />
     * ```
     */
    initial?: Target | VariantLabels;
    /**
     * The React DOM `style` prop, enhanced with support for `MotionValue`s and separate `transform` values.
     *
     * @remarks
     * ```jsx
     * function App() {
     *   const x = useMotionValue(0)
     *
     *   return <motion.div style={{ x, opacity: 1, scale: 0.5 }} />
     * }
     */
    style?: MotionStyle;
    /**
     * By default, Framer Motion generates a `transform` property with a sensible transform order. `transformTemplate`
     * can be used to create a different order, or to append/preprend the automatically generated `transform` property.
     *
     * @remarks
     *
     *
     * @param transform - The latest animated transform props.
     * @param generatedTransform - The transform string as automatically generated by Framer Motion
     *
     * @public
     */
    transformTemplate?(transform: TransformProperties, generatedTransform: string): string;
    /**
     * @internal
     */
    [key: string]: any;
}

/**
 * @public
 */
export declare type MotionStyle = MotionCSS & MotionTransform & MotionCustom;

declare type MotionTransform = MakeMotion<TransformProperties>;

/**
 * @internal
 */
export declare class MotionValue<V = any> {
    private current;
    private prev;
    private timeDelta;
    private lastUpdated;
    private children?;
    private parent?;
    private updateSubscribers?;
    private renderSubscribers?;
    private transformer?;
    private controller?;
    private canTrackVelocity;
    constructor(init: V, { transformer, parent }?: Config<V>);
    addChild(config: Config<V>): MotionValue<V>;
    removeChild(child: MotionValue): void;
    subscribeTo(subscriptions: Set<Subscriber<V>>, subscription: Subscriber<V>): () => boolean;
    addUpdateSubscription(subscription: Subscriber<V>): () => boolean;
    addRenderSubscription(subscription: Subscriber<V>): () => boolean;
    set(v: V, render?: boolean): void;
    notifySubscriber: (subscriber: Subscriber<V>) => void;
    scheduleVelocityCheck: () => import("framesync/lib/types").Process;
    velocityCheck: ({ timestamp }: FrameData) => void;
    setChild: (child: MotionValue<any>) => void;
    get(): V;
    getVelocity(): number;
    control(controller: ActionFactory, config: PopmotionTransitionProps, transformer?: Transformer<V>): Promise<{}>;
    stop(): void;
    destroy(): void;
}

declare class MotionValuesMap {
    private hasMounted;
    private styler;
    private transformTemplate;
    private onUpdate?;
    private values;
    private unsubscribers;
    private isStatic;
    private customStyles?;
    constructor(isStatic: boolean, customStyles?: CustomStyleMap);
    has(key: string): boolean;
    set(key: string, value: MotionValue): void;
    get<Value>(key: string): MotionValue<Value> | undefined;
    get<Value>(key: string, defaultValue: Value): MotionValue<Value>;
    forEach(callback: (value: MotionValue, key: string) => void): void;
    bindValueToStyler(key: string, value: MotionValue): void;
    setOnUpdate(onUpdate?: OnUpdate): void;
    setTransformTemplate(transformTemplate?: TransformTemplate | undefined): void;
    getTransformTemplate(): TransformTemplate | undefined;
    updateTransformTemplate(): void;
    mount(element: Element): void;
    unmount(): void;
}

/**
 * @public
 */
export declare interface None {
    /**
     * Set `type` to `false` for an instant transition.
     *
     * @public
     */
    type: false;
    /**
     * @internal
     */
    from?: number | string;
    /**
     * @internal
     */
    delay?: number;
    /**
     * @internal
     */
    velocity?: number;
}

declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;

declare type Omit_2<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;

declare type OnUpdate = (v: Target) => void;

/**
 * Options for orchestrating the timing of animations.
 *
 * @public
 */
export declare interface Orchestration {
    /**
     * Delay the animation by this duration (in seconds). Defaults to `0`.
     *
     * @remarks
     * ```javascript
     * const transition = {
     *   delay: 0.2
     * }
     * ```
     *
     * @public
     */
    delay?: number;
    /**
     * Describes the relationship between the transition and its children. Set
     * to `false` by default.
     *
     * @remarks
     * When using variants, the transition can be scheduled in relation to its
     * children with either `'beforeChildren'` to finish this transition before
     * starting children transitions, `'afterChildren'` to finish children
     * transitions before starting this transition.
     *
     * ```jsx
     * const container = {
     *   hidden: {
     *     opacity: 0,
     *     // This will ensure all children animations
     *     // finish before this animation starts
     *     transition: { when: 'afterChildren' }
     *   }
     * }
     *
     * const item = {
     *   hidden: { opacity: 0 }
     * }
     *
     * return (
     *   <Frame
     *     variants={container}
     *     animate="hidden"
     *   >
     *     <Frame variants={item} />
     *     <Frame variants={item} />
     *     <Frame variants={item} />
     *   </Frame>
     * )
     * ```
     *
     * @public
     */
    when?: false | "beforeChildren" | "afterChildren";
    /**
     * When using variants, children animations will start after this duration
     * (in seconds).
     *
     * @public
     */
    delayChildren?: number;
    /**
     * When using variants, children animations can be staggered by this
     * duration (in seconds).
     *
     * For instance, if `staggerChildren` is `0.01`, the first child will be
     * delayed by `0` seconds, the second by `0.01`, the third by `0.02` and so
     * on.
     *
     * The calculated stagger delay will be added to `delayChildren`.
     *
     * @public
     */
    staggerChildren?: number;
    /**
     * The direction in which to stagger children.
     *
     * @remarks
     *
     * A value of `1` staggers from the first child to the last while `-1`
     * staggers from the last child to the first.
     *
     * @public
     */
    staggerDirection?: 1 | -1;
}

/**
 * @public
 */
export declare interface PanHandlers {
    /**
     * Callback when the pan gesture is recognised on this element.
     *
     * ```jsx
     * function App() {
     *   const onPan = (event, { point, delta, offset, velocity }) => {
     *      console.log(`Panned to ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onPan={onPan} />
     * }
     * ```
     */
    onPan?(event: MouseEvent | TouchEvent, info: PanInfo): void;
    /**
     * Callback when the pan gesture begins on this element.
     *
     * ```jsx
     * function App() {
     *   const onPanStart = (event, { point, delta, offset, velocity }) => {
     *      console.log(`Started panning at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onPanStart={onPan} />
     * }
     * ```
     */
    onPanStart?(event: MouseEvent | TouchEvent, info: PanInfo): void;
    /**
     * Callback when the pan gesture ends on this element.
     *
     * ```jsx
     * function App() {
     *   const onPanEnd = (event, { point, delta, offset, velocity }) => {
     *      console.log(`Stopped panning at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onPanEnd={onPan} />
     * }
     * ```
     */
    onPanEnd?(event: MouseEvent | TouchEvent, info: PanInfo): void;
}

declare interface PanInfo {
    point: Point;
    delta: Point;
    offset: Point;
    velocity: Point;
}

/**
 * An animation that simulates velocity, acceleration, and friction.
 *
 * @public
 */
export declare interface Physics {
    /**
     * Set `type` to `'physics'` to use an animation that will simulate velocity, friction and acceleration.
     * @public
     */
    type: "physics";
    /**
     * Accelerates `velocity` by this amount every second.
     *
     * ```jsx
     * // `velocity` will be `102` after one second:
     * const transition = {
     *   type: 'physics',
     *   velocity: 100,
     *   acceleration: 2
     * }
     * ```
     *
     * @public
     */
    acceleration?: number;
    /**
     * Amount of friction to apply per frame, from `0` to `1`.
     *
     * `0` is no friction, `1` is a total stop.
     * @public
     */
    friction?: number;
    /**
     * End animation if absolute speed (in units per second) drops below this
     * value. Set to `0.01` by default.
     * @public
     */
    restSpeed?: number;
    /**
     * The value to animate from. By default, this is the current state of the `MotionValue`.
     *
     * @public
     */
    from?: number | string;
    /**
     * The initial velocity of the spring. By default this is the current velocity of the `MotionValue`.
     * @public
     */
    velocity?: number;
    /**
     * @internal
     */
    delay?: number;
}

declare interface Point {
    x: number;
    y: number;
}

declare namespace Point {
    /** @alpha */
    const subtract: (a: Point, b: Point) => Point;
}

declare type PopmotionTransitionProps = Tween | Spring | Keyframes | Physics | Inertia | Just;

/**
 * Creates a server-safe reference to `window`, returning a mock if none is available.
 *
 * @internal
 */
export declare const safeWindow: Window | ServerSafeWindow;

declare interface ScrollMotionValues {
    scrollX: MotionValue<number>;
    scrollY: MotionValue<number>;
    scrollXProgress: MotionValue<number>;
    scrollYProgress: MotionValue<number>;
}

declare interface ServerSafeWindow extends EventTarget {
    onpointerdown: false;
    onpointermove: false;
    onpointerup: false;
    ontouchstart: false;
    ontouchmove: false;
    ontouchend: false;
    onmousedown: false;
    onmousemove: false;
    onmouseup: false;
}

/**
 * An animation that simulates spring physics for realistic motion.
 *
 * This is the default animation for physical values like `x`, `y`, `scale` and `rotate`.
 *
 * @public
 */
export declare interface Spring {
    /**
     * Set `type` to `'spring'` to animate using spring physics for natural
     * movement. Set to `'spring'` by default.
     *
     * @public
     */
    type: "spring";
    /**
     * Stiffness of the spring. Higher values will create more sudden movement.
     * Set to `100` by default.
     * @public
     */
    stiffness?: number;
    /**
     * Strength of opposing force. If set to 0, spring will oscillate
     * indefinitely. Set to `10` by default.
     * @public
     */
    damping?: number;
    /**
     * Mass of the moving object. Higher values will result in more lethargic
     * movement. Set to `1` by default.
     * @public
     */
    mass?: number;
    /**
     * End animation if absolute speed (in units per second) drops below this
     * value and delta is smaller than `restDelta`. Set to `0.01` by default.
     * @public
     */
    restSpeed?: number;
    /**
     * End animation if distance to to is below this value and speed is below
     * `restSpeed`. When animation ends, spring gets “snapped” to to. Set to
     * `0.01` by default.
     * @public
     */
    restDelta?: number;
    /**
     * The value to animate from. By default, this is the current state of the `MotionValue`.
     *
     * @public
     */
    from?: number | string;
    /**
     * @internal
     */
    to?: number | string;
    /**
     * The initial velocity of the spring. By default this is the current velocity of the `MotionValue`.
     * @public
     */
    velocity?: number;
    /**
     * @public
     */
    delay?: number;
}

declare type Subscriber<T> = (v: T) => void;

declare type SVGElements = UnionStringArray<typeof svgElements>;

/**
 * @internal
 */
export declare const svgElements: (keyof ReactSVG)[];

/**
 * Motion-optimised versions of React's SVG components.
 *
 * @public
 */
export declare type SVGMotionComponents = {
    [K in SVGElements]: ComponentType<Omit<SVGAttributes<SVGElement>, "style"> & MotionProps>;
};

/**
 * @public
 */
export declare interface TapHandlers {
    /**
     * Callback when the tap gesture successfully ends on this element.
     *
     * ```jsx
     * function App() {
     *   const onTap = (event, { point }) => {
     *      console.log(`Tapped at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onTap={onPan} />
     * }
     * ```
     */
    onTap?(event: MouseEvent | TouchEvent, info: TapInfo): void;
    /**
     * Callback when the tap gesture starts on this element.
     *
     * ```jsx
     * function App() {
     *   const onTapStart = (event, { point }) => {
     *      console.log(`Tap started at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onTapStart={onPan} />
     * }
     * ```
     */
    onTapStart?(event: MouseEvent | TouchEvent, info: TapInfo): void;
    /**
     * Callback when the tap gesture ends outside this element.
     *
     * ```jsx
     * function App() {
     *   const onTapCancel = (event, { point }) => {
     *      console.log(`Tap cancelled at ${point.x} ${point.y}`)
     *   }
     *
     *   return <motion.div onTapCancel={onPan} />
     * }
     * ```
     */
    onTapCancel?(event: MouseEvent | TouchEvent, info: TapInfo): void;
    /**
     * Properties or variant label to animate to while the component is pressed.
     *
     * ```jsx
     * function App() {
     *   return <motion.div press={{ scale: 0.8 }} />
     * }
     *
     * // With variants
     * function App() {
     *   const variants = {
     *     pressed: { scale: 0.8 }
     *   }
     *
     *   return <motion.div variants={variants} press="pressed" />
     * }
     * ```
     */
    press?: string | TargetAndTransition;
}

declare interface TapInfo {
    point: Point;
}

declare type Target = CSSPropertiesWithoutTransition & TransformProperties & CustomProperties & {
    pathLength?: number;
    pathSpacing?: number;
};

declare type TargetAndTransition = Target & {
    transition?: Transition;
    transitionEnd?: Target;
};

declare type TargetResolver = (props: any, current: Target, velocity: Target) => TargetAndTransition;

declare type Transformer<T> = (v: T) => T;

declare type Transformer_2 = (v: any) => any;

declare interface TransformProperties {
    x?: string | number;
    y?: string | number;
    z?: string | number;
    rotate?: string | number;
    rotateX?: string | number;
    rotateY?: string | number;
    rotateZ?: string | number;
    scale?: string | number;
    scaleX?: string | number;
    scaleY?: string | number;
    scaleZ?: string | number;
    skew?: string | number;
    skewX?: string | number;
    skewY?: string | number;
    originX?: string | number;
    originY?: string | number;
    originZ?: string | number;
    perspective?: string | number;
}

declare type TransformTemplate = (transform: TransformProperties, generatedTransform: string) => string;

declare type Transition = (Orchestration & TransitionDefinition) | (Orchestration & TransitionMap);

/**
 * @public
 */
declare type TransitionDefinition = Tween | Spring | Keyframes | Physics | Inertia | Just | None;

declare type TransitionMap = Orchestration & {
    [key: string]: TransitionDefinition;
};

/**
 * An animation that animates between two values over a specific duration of time.
 *
 * This is the default animation for non-physical values like `color` and `opacity`.
 *
 * @public
 */
export declare interface Tween {
    /**
     * Set `type` to `'tween'` to use a duration-based tween animation.
     *
     * @remarks
     * If any non-orchestration `transition` values are set without a `type` prop,
     * "tween" is used as the default animation.
     *
     * @public
     */
    type?: "tween";
    /**
     * The duration of the tween animation. Set to `0.3` by default.
     *
     * @public
     */
    duration?: number;
    /**
     * The easing function to use. Set either as:
     *
     *   - The name of an inbuilt easing function,
     *   - An array of four numbers to define a cubic bezier curve,
     *   - An easing function, that accepts and returns a value `0-1`.
     *
     * ```jsx
     * const transition = {
     *   ease: [0.17, 0.67, 0.83, 0.67]
     * }
     * ```
     *
     * @public
     */
    ease?: [number, number, number, number] | "linear" | "easeIn" | "easeOut" | "easeInOut" | "circIn" | "circOut" | "circInOut" | "backIn" | "backOut" | "backInOut" | "anticipate" | EasingFunction;
    /**
     * The duration of time already elapsed in the animation. Set to `0` by
     * default.
     *
     * @internal
     */
    elapsed?: number;
    /**
     * The number of times to loop the animation.
     *
     * Set to `Infinity` for perpetual looping.
     *
     * @public
     */
    loop?: number;
    /**
     * The number of times to flip the animation by swapping the `to` and `from` values.
     *
     * Set to `Infinity` for perpetual flipping.
     *
     * @public
     */
    flip?: number;
    /**
     * The number of times to reverse the animation.
     *
     * Set to `Infinity` for perpetual reversing.
     *
     * @public
     */
    yoyo?: number;
    /**
     * The value to animate from. By default, this is the current state of the `MotionValue`.
     *
     * @public
     */
    from?: number | string;
    /**
     * @internal
     */
    to?: number | string;
    /**
     * @internal
     */
    velocity?: number;
    /**
     * @internal
     */
    delay?: number;
}

declare type UnionStringArray<T extends string[]> = T[number];

declare type UnwrapFactory<F> = F extends DetailedHTMLFactory<infer P, any> ? P : never;

/**
 * If the provided value is a MotionValue, this returns the actual value, otherwise just the value itself
 *
 * @internal
 */
export declare function unwrapMotionValue<V>(value: V | MotionValue<V>): V;

/**
 * Manually start, stop and sequence animations on one or more `motion` components.
 *
 * @param variants - An optional named map of variants.
 * @param defaultTransition - An optional default transition to use when a variant doesn’t have an explicit `transition` property set.
 * @returns Animation controller with `start` and `stop` methods
 *
 * @public
 */
export declare function useAnimation(variants?: Variants, defaultTransition?: Transition): AnimationControls;

/**
 * Cycles through a series of states. Can be used to toggle between or cycle through animations.
 *
 * ```jsx
 * function App() {
 *   const [x, cycleX] = useCycle([0, 100, 200])
 *
 *   return <Frame animate={{ x: x }} onClick={() => cycleX()} />
 * }
 * ```
 *
 * @param items - An array of the possible states
 * @param initialIndex - Index of initial state. Defaults to `0`
 * @returns [currentState, cycleState]
 *
 * @public
 */
export declare function useCycle<T>(items: T[], initialIndex?: number): CycleState<T>;

/**
 * Uses the ref that is passed in, or creates a new one
 * @param external - External ref
 * @internal
 */
export declare function useExternalRef<E = Element>(external?: Ref<E | null>): RefObject<E | null>;

/**
 * Add pan and tap gesture recognition to an element.
 *
 * @param props - Gesture event handlers
 * @param ref - React `ref` containing a DOM `Element`
 * @public
 */
export declare function useGestures<GestureHandlers>(props: GestureHandlers, ref: RefObject<Element>): void;

/**
 * Creates a `MotionValue` to track the state and velocity of a value.
 *
 * Usually, these are created automatically. For advanced use-cases, like use with `useTransformedValue`, you can create `MotionValue`s externally and pass them into the animated component via the `style` prop.
 *
 * ```jsx
 * function App() {
 *   const scale = useMotionValue(1)
 *
 *   return <Frame style={{ scale: scale }} />
 * }
 * ```
 *
 * @param initial - The initial state.
 *
 * @public
 */
export declare function useMotionValue<T>(initial: T): MotionValue<T>;

/**
 *
 * @param handlers
 * @param ref
 * @internal
 */
export declare function usePanGesture(handlers: PanHandlers, ref: RefObject<Element>): undefined;

export declare function usePanGesture(handlers: PanHandlers): {
    onPointerDown: EventHandler;
};

/**
 * @param handlers
 * @internal
 */
export declare function useTapGesture(handlers: TapHandlers & ControlsProp): {
    onPointerDown: EventHandler;
};

export declare function useTapGesture(handlers: TapHandlers & ControlsProp, ref: RefObject<Element>): undefined;

/**
 * Create a `MotionValue` that transforms the output of another `MotionValue` through a function.
 *
 * ```jsx
 * const double = (v) => v * 2
 *
 * function App() {
 *   const x = useMotionValue(0)
 *   const y = useTransformedValue(x, double)
 *   // y will be x, doubled
 *
 *   return <Frame style={{ x, y }} />
 * }
 * ```
 *
 * @param value - The `MotionValue` to transform the output of.
 * @param transform - Function that accepts the output of `value` and returns a new value.
 * @returns `MotionValue`
 *
 * @public
 */
export declare function useTransformedValue(value: MotionValue, transform: Transformer_2): MotionValue;

/**
 * Create a `MotionValue` that transforms the output of another `MotionValue` by mapping it from one range of values into another.
 *
 * @remarks
 *
 * Given an input range of `[-200, -100, 100, 200]` and an output range of
 * `[0, 1, 1, 0]`, the returned `MotionValue` will:
 *
 * - When provided a value between `-200` and `-100`, will return a value between `0` and  `1`
 * - When provided a value between `-100` and `100`, will return `1`
 * - When provided a value between `100` and `200, will return a value between `1` and  `0`
 *
 * The input range must be a linear series of numbers. The output range
 * can be any value type supported by Framer Motion: numbers, colors, shadows, etc.
 *
 * ```jsx
 * function App() {
 *   const x = useMotionValue(0)
 *   const xRange = [-200, -100, 100, 200]
 *   const opacityRange = [0, 1, 1, 0]
 *   const opacity = useTransformedValue(x, xRange, opacityRange)
 *
 *   return <Frame dragEnabled="x" style={{ opacity, x }} />
 * }
 * ```
 *
 * @param inputValue - `MotionValue`
 * @param inputRange - A linear series of numbers (either all increasing or decreasing)
 * @param outputRange - A series of numbers, colors or strings. Must be the same length as `from`.
 * @param options -
 *
 *  - clamp: boolean - Clamp values to within the given range. Defaults to `true`
 *
 * @returns `MotionValue`
 *
 * @public
 */
export declare function useTransformedValue(value: MotionValue<number>, from: number[], to: any[], options?: MapOptions): MotionValue;

/**
 * Provides `MotionValue`s that update when the viewport scrolls.
 *
 * - Test
 * - Test 2
 *
 * ```jsx
 * function App() {
 *   const { scrollYProgress } = useViewportScrollValues()
 *   const scaleX = useTransformedValue(scrollYProgress, [0, 1], [0, 1])
 *
 *   return <Frame style={{ scaleX }} />
 * }
 * ```
 *
 * @public
 */
export declare function useViewportScrollValues(): ScrollMotionValues;

declare type Variant = TargetAndTransition | TargetResolver;

declare type VariantLabels = string | string[];

declare type Variants = {
    [key: string]: Variant;
};

export { }
